/****************************************************************************
 Copyright (c) 2013-2016 Chukong Technologies Inc.
 Copyright (c) 2017-2023 Xiamen Yaji Software Co., Ltd.

 http://www.cocos.com

 Permission is hereby granted, free of charge, to any person obtaining a copy
 of this software and associated documentation files (the "Software"), to deal
 in the Software without restriction, including without limitation the rights to
 use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 of the Software, and to permit persons to whom the Software is furnished to do so,
 subject to the following conditions:

 The above copyright notice and this permission notice shall be included in
 all copies or substantial portions of the Software.

 THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 THE SOFTWARE.
****************************************************************************/

#ifndef HTTP_COOKIE_H
#define HTTP_COOKIE_H
/// @cond DO_NOT_SHOW

#include "base/std/container/string.h"
#include "base/std/container/vector.h"

struct CookiesInfo {
    ccstd::string domain;
    bool tailmatch;
    ccstd::string path;
    bool secure;
    ccstd::string name;
    ccstd::string value;
    ccstd::string expires;
};

class HttpCookie {
public:
    void readFile();

    void writeFile();
    void setCookieFileName(const ccstd::string &fileName);

    const ccstd::vector<CookiesInfo> *getCookies() const;
    const CookiesInfo *getMatchCookie(const ccstd::string &url) const;
    void updateOrAddCookie(CookiesInfo *cookie);

private:
    ccstd::string _cookieFileName;
    ccstd::vector<CookiesInfo> _cookies;
};

/// @endcond
#endif /* HTTP_COOKIE_H */
